#!/bin/bash
#     kde-l10n-synk.sh – Synkronisering av omsettingar frå og til KDE.
#     Copyright © 2007–2008 Karl Ove Hufthammer <karl@huftis.org>.
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

## MÅ FIKSAST: Oppdatering av «docs/» i vår SVN (bilde og sånt).

#Ubaci datum
echo ''
echo '##############################'
date
echo '##############################'
echo ''

# CRONTAB
LANG=en_US.UTF-8
export LANG

export KDEREPO=/home/zpintar/KDE_HR/kde_svn # sadrži checkout sa KDE summit-a, stable i branch foldera sa KDE SVN repoa
export SLNREPO=/home/zpintar/KDE_HR/kde-croatia # folder koji sadrži podatke o prevodima sa SF SVN-a
export KOMP="$SLNREPO/skript/compendium" # mjesto gdje su pohranjene compendium datoteke
versjon="kde4/summit"
export PATH=$SLNREPO/$versjon/pology/scripts:$PATH
sprak="hr" # jezik - za naše potrebe će biti hr
hjelpetekstar="nei" # Dokumentacija. Kako ne prevodimo dokumentaciju, nego samo GUI, treba ostati "nei"
kjapp="nei" # Brzina merganja. Samo povremeno treba prebaciti na "nei" (jednom mjesečno npr.)

if [ $kjapp = "ja" ]
then
  operasjon="merge,scatter"
else
  operasjon="merge,scatter"
fi
if [ $hjelpetekstar = "ja" ]
then
  mapper="summit/messages summit/docmessages"
else
  mapper="summit/messages"
fi


function leggtilfjern
{
  nye=$(svn status | grep "^\?" | cut -c8-)
  fjerna=$(svn status | grep "^\!" | cut -c8-)
  if [ -n "$nye" ]; then svn add $nye; fi
  if [ -n "$fjerna" ]; then svn delete $fjerna; fi
}


function compendium # kreira compendium za automatsko prevođenje kada se isti stringovi ponavljaju u više datoteka
{
# Update podataka sa SF SVN-a
echo -e "\n*** Update podataka sa SF SVN-a ***\n"
cd $SLNREPO/$versjon

ExitCode=1
Errcount=0
while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
svn up
  ExitCode=$?
  let Errcount++
done 

mkdir -p "$KOMP"

# Kreiranje compendiuma
  for sprk in $sprak
  do
      echo -e "\n*** Kreiram compendium za jezik $sprk ***\n"
      cd $SLNREPO/$versjon/$sprk/summit/messages
      # First output all messages.
      find $SLNREPO/$versjon/$sprk -name '*.po' -print0 | xargs -0 msgcat --use-first --output-file="$KOMP/${sprk}.po"
      # Also include obsolete messages.
      msgattrib --clear-obsolete --output-file="$KOMP/${sprk}.po" "$KOMP/${sprk}.po"
      # Mark all messages as fuzzy, so they will be merged as fuzzy.
      msgattrib --translated --set-fuzzy --no-location --no-wrap --output-file="$KOMP/${sprk}.po" "$KOMP/${sprk}.po"

      # Remove any ~ accelerators (from OpenOffice.org PO files).
      cat "$KOMP/${sprk}.po" | sed 's/~\(\w\)/\&\1/g' > "$KOMP/${sprk}.tmp"

      # Remove any translator comments and source references.
      grep -v '^#[. ]' "$KOMP/${sprk}.tmp" | msguniq --use-first --no-wrap | \
      grep -v '^#[. ]' > "$KOMP/${sprk}.po"

      # Remove temporary files.
      rm -f "$KOMP/"*.tmp

      # Remove any NAME OF TRANSLATORS and EMAIL OF TRANSLATOR strings      
      sed -i '/^#, fuzzy$/{n;/^msgctxt "[^ ]\+ OF TRANSLATORS"$/{n;n;s/".*"/""/}}' "$KOMP/${sprk}.po"
  done
} # Kraj kreiranja

function update_po # Glavna funkcija. Koristi se za update PO datoteka (move/rename/delete, ako se takve stvari dese i na KDE SVN repou). merganje...
{
# Update podataka sa SF SVN-a
echo -e "\n*** Update podataka sa SF SVN-a ***\n"
cd $SLNREPO/$versjon

ExitCode=1
Errcount=0
while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
svn up
  ExitCode=$?
  let Errcount++
done 


# Stvari koje sam zaboravio poslati
  for sprk in $sprak
  do
    for mappe in $mapper
    do
      cd $SLNREPO/$versjon/$sprk/$mappe
      ExitCode=1
      Errcount=0
      while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
        svn commit -m "SF-KDE: Još prevoda."
        ExitCode=$?
        let Errcount++
      done 
      leggtilfjern
      ExitCode=1
      Errcount=0
      while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
        svn commit -m "SF-KDE: Novi prevod."
        ExitCode=$?
        let Errcount++
      done     
    done
  done

# Commit podataka na SF SVN
  cd $SLNREPO/$versjon
  # Move/delete/rename files that have been moved/deleted/renamed in KDE.
  # (Noisy, so suppress the output.)
  echo -e "\n*** Move/delete/rename kao što se dešava na KDE SVN-u ***\n"
  bash ./scripts/process_orphans.sh >/dev/null
  for sprk in $sprak
  do
    if [ $sprk = "hr" ]; then
        langsprk="Croatian"
   fi
    find $sprk -name '*.po' | while read fil
    do
      namn=$(basename ${fil%.po})
      sed -i -r -e '1,/^msg/{/^#[^@]*$/d}' -e '1,/^msg/{/^#[^>]*$/d}' -e "1,1s/(.*)/# Translation of $namn to $langsprk\n#\n\1/" $fil
    done
  done
  find . -name '*.po' | xargs sed -i -e '/^#, fuzzy$/{n;/^msgctxt "[^ ]\+ OF TRANSLATORS"$/{n;n;s/".*"/""/}}' \
                                     -e '/^msgctxt "[^ ]\+ OF TRANSLATORS"$/{n;n;s/ ?, ?/,/g;s/ "$/"/g}'
  
  echo -e "\n*** Provjera datoteka ***\n"
  bash $SLNREPO/skript/check_po_files
  posieve.py set_header -screate -s'field:Language-Team:Croatian <kde-croatia-list@lists.sourceforge.net>' hr
  # posieve.py set_header -screate -s'field:Plural-Forms:nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%' hr
  # Normalize the headers, and add some useful header lines.
  posieve.py normalize_header,equip-header-tp-kde hr
  ExitCode=1
  Errcount=0
  while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
    svn commit -m 'SF-KDE: Automatsko ažuriranje (move/delete, patching i sl.).'
    ExitCode=$?
    let Errcount++
  done 
for sprk in $sprak
  do
    cd $SLNREPO/$versjon/$sprk
    echo -e "\n*** Update datoteka za $versjon i jezik $sprk ***\n"
    for mappe in $mapper
    do
      cd $SLNREPO/$versjon/$sprk/$mappe
      echo "--- Debugiranje za $mappe i jezik $sprk ---"
      find -name "*.po" -type f | while read fil
      do
          endra=$(sed -f $SLNREPO/skript/feilfiksar $fil)
          echo "$endra" > $fil
          if [ "$fil" = './kdeedu/blinken.po' ]; then # I blinken skal ikkje «...» endrast til «…».
            sed -i 's/…/.../g' $fil
          fi
      done # Završetak .po-fil
      ExitCode=1
      Errcount=0
      while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
        svn commit -m "SF-KDE: Automatsko formatiranje i korekcija čestih grešaka."
        ExitCode=$?
        let Errcount++
      done 
      echo "--- Merganje za $mappe i jezik $sprk ---"
      find -name "*.po" -type f | while read po
      do
            pot=../../../templates/$mappe/${po%.po}.pot
                datopo=$(grep '^"POT-Creation-Date:' $po)
                datopot=$(grep '^"POT-Creation-Date:' $pot)
        if [[ -e "$pot" && ("$datopo" != "$datopot" || "$kjapp" != "ja") ]]; then
                              echo "$po se mora ažurirati:"
            msgmerge --verbose --update --previous --backup=none --compendium="$KOMP/${sprk}.po" --silent $po $pot
            porewrap.py --no-wrap $po
        fi
      done # Završetak .po-fil
    # Provjera svih grešaka i ispis
    echo -e "\n*** Izvještaj o neriješenim greškama u .po datotekama ***\n"
    posieve.py --skip-obsolete -c check-tp-kde $SLNREPO/$versjon/$sprk/$mappe
    done # Završetak foldera
  done # Završetak jezika
  cd $SLNREPO/$versjon
  ExitCode=1
  Errcount=0
  while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
    svn commit -m "SF-KDE: Ažuriranje datoteka bazirano na zadnjim template datotekama."
    ExitCode=$?
    let Errcount++
  done 
} # Kraj oppdater_po


function commit_po # Commita prevode na KDE SVN repo, te vrši scatter summit datoteka u stable+trunk datoteke.
{
    rsync_val="--exclude-from=$SLNREPO/skript/ekskluderingsfil --recursive --delete"

    echo -e "\n*** Dohvat datoteka, templatea i skripti ***\n"
    ExitCode=1
    Errcount=0
    while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
      svn up $KDEREPO/{trunk,branches/stable}/l10n-kde4/{scripts,templates,hr}
      ExitCode=$?
      let Errcount++
    done 
    ExitCode=1
    Errcount=0
    while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
      svn up $KDEREPO/trunk/l10n-support/{scripts,templates,pology,hr}
      ExitCode=$?
      let Errcount++
    done 
    ExitCode=1
    Errcount=0
    while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
      svn up -N $KDEREPO/trunk/l10n-support/
      ExitCode=$?
      let Errcount++
    done 
    # Premještanje i ostalo:
    echo -e "\n*** Move/delete/rename kao što se dešava na KDE SVN-u ***\n"
    cd $KDEREPO/trunk/l10n-kde4
    bash ./scripts/process_orphans.sh >/dev/null
    cd $KDEREPO/branches/stable/l10n-kde4
    bash ./scripts/process_orphans.sh >/dev/null

    for sprk in $sprak
    do
      echo -e "\n*** Dodaj/Briši za $versjon i jezik $sprk u KDEREPO ***\n"
      cd $KDEREPO/trunk/l10n-support/$sprk
      rsync $rsync_val $SLNREPO/$versjon/$sprk/ $KDEREPO/trunk/l10n-support/$sprk
      leggtilfjern
      if [ $sprk = "hr" ]; then
          langsprk="Croatian"
     fi
      cd $KDEREPO/trunk/l10n-support/
      posummit.py --force scripts/messages.summit $sprk $operasjon trunk: stable:
      if [ $hjelpetekstar = "ja" ]
      then
        posummit.py --force scripts/docmessages.summit $sprk $operasjon trunk: stable:
      fi
      
      cd $KDEREPO/trunk/l10n-support/$sprk
      leggtilfjern
      cd ..
      ExitCode=1
      Errcount=0
      while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
        svn commit -m "Updated $langsprk translations for summit." $sprk
        ExitCode=$?
        let Errcount++
      done    
      cd $KDEREPO/trunk/l10n-kde4/$sprk
      #rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/data .
      #rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/scripts .
      leggtilfjern
      cd ..
      ExitCode=1
      Errcount=0
      while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
        svn commit -m "Updated $langsprk translations for kde4-trunk." $sprk
        ExitCode=$?
        let Errcount++
      done    
       
      cd $KDEREPO/branches/stable/l10n-kde4/$sprk
      #rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/data .
      #rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/scripts .
      leggtilfjern
      cd ..
      ExitCode=1
      Errcount=0
      while [[ $ExitCode != 0 && $Errcount < 5 ]]; do
        svn commit -m "Updated $langsprk translations for kde4-stable." $sprk
        ExitCode=$?
        let Errcount++
      done    
     done
} # Kraj tilbake_po

function lag_dok # Funkcija za rad sa dokumentacijom (nebitna za nas)
{
    svn up $KDEREPO/{trunk,branches/stable}/l10n-kde4/{documentation,*/docs}
    for sprk in $sprak
    do
      echo -e "\n*** Lagar hjelpefiler for trunk og språk $sprk ***\n"
      cd $KDEREPO/trunk/l10n-kde4
      ./scripts/update_xml $sprk
      svn commit -m "Updated $langsprk translations for documentation (kde4-trunk)." $sprk/docs

      echo -e "\n*** Lagar hjelpefiler for stable og språk $sprk ***\n"
      cd $KDEREPO/branches/stable/l10n-kde4
      ./scripts/update_xml $sprk
      svn commit -m "Updated $langsprk translations for documentation (kde4-stable)." $sprk/docs
    done
} # Slutt tilbake_po

echo -e "kde-l10n-synk.sh\n\
    Copyright © 2007 Karl Ove Hufthammer <karl@huftis.org>.
    This program comes with ABSOLUTELY NO WARRANTY; for details,
    see the source code. This is free software, and you are welcome
    to redistribute it under certain conditions; again, see the
    source code for details.\n"

val=$1
if [ -z "$val" ]; then
  echo -e "Funkcija:\n\
  update_po      Ažuriranje SF SVN-a sa svim potrebnim radnjama.\n\
  compendium     Kreiranje compendiuma.
  commit_po      Transfer datoteka natrag na KDE SVN.\n\
  lag_dok        Kreiranje i slanje dokumentacije na KDE SVN.\n\
  all            Učini sve radnje (po redu)."
elif [ $val = all ]; then
  compendium
  update_po
  commit_po
cat $SLNREPO/../crontab.log | mail -s "Datum: $(date)" zarko.pintar@gmail.com
if [ $hjelpetekstar = "ja" ]
then
  lag_dok
fi
else
  for i in $@; do
    echo -e "Funkcija «$i».\n"
    $i
  done
fi
