#!/bin/bash
#     kde-l10n-synk.sh – Synkronisering av omsettingar frå og til KDE.
#     Copyright © 2007–2008 Karl Ove Hufthammer <karl@huftis.org>.
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

## MÅ FIKSAST: Oppdatering av «docs/» i vår SVN (bilde og sånt).

export KDEREPO=~/kde_svn/svn # sadrži checkout sa KDE summit-a, stable i branch foldera sa KDE SVN repoa
export SLNREPO=~/sf_svn/svn # folder koji sadrži podatke o prevodima sa SF SVN-a
export KOMP="$SLNREPO/skript/compendium" # mjesto gdje su pohranjene compendium datoteke
versjon="kde4/summit"
export PATH=$SLNREPO/$versjon/pology/scripts:$PATH
sprak="hr" # jezik - za naše potrebe će biti hr
hjelpetekstar="nei" # Dokumentacija. Kako ne prevodimo dokumentaciju, nego samo GUI, treba ostati "nei"
kjapp="ja" # Brzina merganja. Samo povremeno treba prebaciti na "nei" (jednom mjesečno npr.)

if [ $kjapp = "ja" ]
then
  operasjon="scatter,merge"
else
  operasjon="scatter,merge"
fi
if [ $hjelpetekstar = "ja" ]
then
  mapper="summit/messages summit/docmessages"
else
  mapper="summit/messages"
fi


function leggtilfjern
{
  nye=$(svn status | grep "^\?" | cut -c8-)
  fjerna=$(svn status | grep "^\!" | cut -c8-)
  if [ -n "$nye" ]; then svn add $nye; fi
  if [ -n "$fjerna" ]; then svn delete $fjerna; fi
}

function oppdater_pot #više se ne koristi prema rječima Norvežana
{
# Oppdater frå KDE-SVN
echo "*** Hentar nyaste malfiler ***"
    cd $SLNREPO/$versjon
    svn up templates
} # Slutt oppdater_pot



function compendium # kreira compendium za automatsko prevođenje kada se isti stringovi ponavljaju u više datoteka
{
# Update podataka sa SF SVN-a
echo -e "\n*** Update podataka sa SF SVN-a ***\n"
cd $SLNREPO/$versjon
svn up

mkdir -p "$KOMP"

# Kreiranje compendiuma
  for sprk in $sprak
  do
      echo -e "\n*** Kreiram compendium za jezik $sprk ***\n"
      cd $SLNREPO/$versjon/$sprk/summit/messages
      find $SLNREPO/$versjon/$sprk -name '*.po' -print0 | xargs -0 msgcat --use-first --output-file="$KOMP/${sprk}.po"
      msgattrib --clear-obsolete --output-file="$KOMP/${sprk}.po" "$KOMP/${sprk}.po"
      msgattrib --translated --set-fuzzy --no-location --no-wrap --output-file="$KOMP/${sprk}.po" "$KOMP/${sprk}.po"
      sed -i '/^#, fuzzy$/{n;/^msgctxt "[^ ]\+ OF TRANSLATORS"$/{n;n;s/".*"/""/}}' "$KOMP/${sprk}.po"
  done
} # Kraj kreiranja



function hent_fra_kde3 # Ne koristi se više
{
# Henter gamle omsettingsfiler frå KDE 3, og overfører dei til KDE 4.
echo "*** Overfører eventuelle omsettingsfiler frå KDE 3 til KDE 4 ***"
    cd $SLNREPO/$versjon
    svn up templates
    for sprk in $sprak
    do
      svn up $sprk
      for mappe in $mapper
      do
        cd $SLNREPO/$versjon/templates
        find $mappe -name '*.pot' | while read pot
        do
          po=${pot%t}
          fil=$(basename $po)
          adresse=$(dirname $po)
          for kde3vers in kde3-stable kde3-trunk
          do
            if [ ! -e ../$sprk/$po ]
              then
                nyfil=$(find $SLNREPO/$kde3vers/$sprk/$(basename $(echo $mappe)) -name "$fil")
              if [ ! -z "$nyfil" ]
              then
                echo "Kopierer omsetting av $fil frå $kde3vers ($nyfil)."
                mkdir -p "$(dirname "$SLNREPO/$versjon/$sprk/$adresse/$fil")"
                svn add "$(dirname "$SLNREPO/$versjon/$sprk/$adresse/$fil")"
                svn copy $nyfil $SLNREPO/$versjon/$sprk/$adresse/$fil
              fi
            fi
          done # for kde3vers
        done # find $mappe
      done # for mappe
    done # for sprk
    svn commit -m "Overført gamle omsettingar frå KDE 3." $SLNREPO/$versjon/{hr}
} # Slutt hent_fra_kde3


function update_po # Glavna funkcija. Koristi se za update PO datoteka (move/rename/delete, ako se takve stvari dese i na KDE SVN repou). merganje...
{
# Update podataka sa SF SVN-a
echo -e "\n*** Update podataka sa SF SVN-a ***\n"
cd $SLNREPO/$versjon
svn up

# Stvari koje sam zaboravio poslati
  for sprk in $sprak
  do
    for mappe in $mapper
    do
      cd $SLNREPO/$versjon/$sprk/$mappe
      svn commit -m "SF-KDE: Jos prevoda."
      leggtilfjern
      svn commit -m "SF-KDE: Novi prevod."
    done
  done

# Commit podataka na SF SVN
  cd $SLNREPO/$versjon
  bash ./scripts/process_orphans.sh
  for sprk in $sprak
  do
    if [ $sprk = "hr" ]; then
        langsprk="Croatian"
   fi
    find $sprk -name '*.po' | while read fil
    do
      namn=$(basename ${fil%.po})
      sed -i -r -e '1,/^msg/{/^#[^@]*$/d}' -e '1,/^msg/{/^#[^>]*$/d}' -e "1,1s/(.*)/# Translation of $namn to $langsprk\n#\n\1/" $fil
    done
  done
  find . -name '*.po' | xargs sed -i -e '/^#, fuzzy$/{n;/^msgctxt "[^ ]\+ OF TRANSLATORS"$/{n;n;s/".*"/""/}}' \
                                     -e '/^msgctxt "[^ ]\+ OF TRANSLATORS"$/{n;n;s/ ?, ?/,/g;s/ "$/"/g}'
  bash $SLNREPO/skript/check_po_files
  posieve.py set_header -screate -s'field:Language-Team:Croatian <kde-croatia-list@lists.sourceforge.net>' hr
  # posieve.py set_header -screate -s'field:Plural-Forms:nplurals=3; plural=(n%10==1 && n%100!=11 ? 0 : n%10>=2 && n%' hr
  posieve.py normalize_header hr
  svn commit -m 'SF-KDE: Automatsko azuriranje (move/delete, patching i sl.).'
  for sprk in $sprak
  do
    cd $SLNREPO/$versjon/$sprk
    echo -e "\n*** Update datoteka za $versjon i jezik $sprk ***\n"
    for mappe in $mapper
    do
      cd $SLNREPO/$versjon/$sprk/$mappe
      echo "--- Debugiranje za $mappe i jezik $sprk ---"
      find -name "*.po" -type f | while read fil
      do
          endra=$(sed -f $SLNREPO/skript/feilfiksar $fil)
          echo "$endra" > $fil
          if [ "$fil" = './kdeedu/blinken.po' ]; then # I blinken skal ikkje «...» endrast til «…».
            sed -i 's/…/.../g' $fil
          fi
      done # Završetak .po-fil
      svn commit -m "SF-KDE: Automatsko formatiranje i korekcija cestih gresaka."

      echo "--- Update za $mappe i jezik $sprk ---"
      find -name "*.po" -type f | while read po
      do
            pot=../../../templates/$mappe/${po%.po}.pot
                datopo=$(grep '^"POT-Creation-Date:' $po)
                datopot=$(grep '^"POT-Creation-Date:' $pot)
        if [[ -e "$pot" && ("$datopo" != "$datopot" || "$kjapp" != "ja") ]]; then
                              echo "$po se mora ažurirati:"
            msgmerge --verbose --update --previous --backup=none --compendium "$KOMP/${sprk}.po" --silent $po $pot
            porewrap --no-wrap $po
        fi
      done # Završetak .po-fil
    done # Završetak foldera
  done # Završetak jezika
  cd $SLNREPO/$versjon
  svn commit -m "SF-KDE: Azuriranje datoteka bazirano na zadnjim template datotekama."
} # Kraj oppdater_po


function magic_po
{
echo -e "\n*** Update podataka sa SF SVN-a ***\n"
cd $SLNREPO/$versjon
svn up
# Språvask omsettingsfiler.
# Førebels har berre nynorsk har ei fornuftig språkvaskingsfil.
  for sprk in nn
  do
    echo -e "\n*** Språkvaskar omsettingsfiler for $versjon og språk $sprk ***\n"
    cd $SLNREPO/$versjon/$sprk
    find -name "*.po" -type f | while read fil
     do
       $SLNREPO/skript/magicpo/magicpo --silent --nomarkfuzzy --nolasttrans -f $SLNREPO/openoffice/utils/MagicPO/dictionaries/magicpo-${sprk}-spraakvask.dict $fil
       msgmerge --silent --backup=none --previous -U $fil $SLNREPO/$versjon/templates/${fil}t
     done # Ferdig med .po-fil
  done # Slutt sprk
  cd $SLNREPO/$versjon
  svn commit -m "SF-KDE: Automatisk språkvask ved hjelp av MagicPO."
} # Slutt magic_po

function commit_po # Commita prevode na KDE SVN repo, te vrši scatter summit datoteka u stable+trunk datoteke.
{
    rsync_val="--exclude-from=$SLNREPO/skript/ekskluderingsfil --recursive --delete"

    echo -e "\n*** Dohvat datoeka, templatea i skripti ***\n"
    svn up $KDEREPO/{trunk,branches/stable}/l10n-kde4/{scripts,templates,hr}
    svn up $KDEREPO/trunk/l10n-support/{scripts,templates,pology,hr}
    svn up -N $KDEREPO/trunk/l10n-support/

    # Premještanje i ostalo:
    cd $KDEREPO/trunk/l10n-kde4
    bash ./scripts/process_orphans.sh
    cd $KDEREPO/branches/stable/l10n-kde4
    bash ./scripts/process_orphans.sh

    for sprk in $sprak
    do
      echo -e "\n*** Dodaj/Briši za $versjon i jezik $sprk u KDEREPO ***\n"
      cd $KDEREPO/trunk/l10n-support/$sprk
      rsync $rsync_val $SLNREPO/$versjon/$sprk/ $KDEREPO/trunk/l10n-support/$sprk
      if [ $sprk = "hr" ]; then
        find . -name '*.po' -type f -print0 | xargs -0 sed -r -i -f $SLNREPO/skript/formfeil
      fi
      leggtilfjern
      if [ $sprk = "hr" ]; then
          langsprk="Croatian"
     fi
      cd $KDEREPO/trunk/l10n-support/
      posummit.py scripts/messages.summit $sprk $operasjon
      if [ $hjelpetekstar = "ja" ]
      then
        posummit.py scripts/docmessages.summit $sprk $operasjon
      fi
      
      cd $KDEREPO/trunk/l10n-support/$sprk
      leggtilfjern
      cd ..
      svn commit -m "Updated $langsprk translations for summit." $sprk
      
      cd $KDEREPO/trunk/l10n-kde4/$sprk
      #rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/data .
      #rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/scripts .
      leggtilfjern
      cd ..
      svn commit -m "Updated $langsprk translations for kde4-trunk." $sprk
      
      cd $KDEREPO/branches/stable/l10n-kde4/$sprk
      #rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/data .
      #rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/scripts .
      leggtilfjern
      cd ..
      svn commit -m "Updated $langsprk translations for kde4-stable." $sprk
    done
} # Kraj tilbake_po

function lag_dok # Funkcija za rad sa dokumentacijom (nebitna za nas)
{
    svn up $KDEREPO/{trunk,branches/stable}/l10n-kde4/{documentation,*/docs}
    for sprk in $sprak
    do
      echo -e "\n*** Lagar hjelpefiler for trunk og språk $sprk ***\n"
      cd $KDEREPO/trunk/l10n-kde4
      ./scripts/update_xml $sprk
      svn commit -m "Updated $langsprk translations for documentation (kde4-trunk)." $sprk/docs

      echo -e "\n*** Lagar hjelpefiler for stable og språk $sprk ***\n"
      cd $KDEREPO/branches/stable/l10n-kde4
      ./scripts/update_xml $sprk
      svn commit -m "Updated $langsprk translations for documentation (kde4-stable)." $sprk/docs
    done
} # Slutt tilbake_po

echo -e "kde-l10n-synk.sh\n\
    Copyright © 2007 Karl Ove Hufthammer <karl@huftis.org>.
    This program comes with ABSOLUTELY NO WARRANTY; for details,
    see the source code. This is free software, and you are welcome
    to redistribute it under certain conditions; again, see the
    source code for details.\n"

val=$1
if [ -z "$val" ]; then
  echo -e "Opis:\n\
  oppdater_pot   Update svih predložaka sa KDE SVN-a u SF SVN direktorij.\n\
  hent_fra_kde3  Dohvat datoteka sa KDE3 i transfer u KDE 4.\n\
  oppdater_po    Ažuriranje SF SVN-a sa svim potrebnim radnjama.\n\
  lag_kompendium Kreiranje compendiuma.
  magic_po       Nemam pojma, ali se ne koristi više.\n\
  tilbake_po     Transfer datoteka natrag na KDE SVN.\n\
  lag_dok        Kreiranje i slanje dokumentacije na KDE SVN.\n\
  alt            Učini sve radnje (po redu)."
elif [ $val = alt ]; then
#  oppdater_pot
  lag_kompendium
  oppdater_po
#  hent_fra_kde3
#  magic_po
  tilbake_po
if [ $hjelpetekstar = "ja" ]
then
  lag_dok
fi
else
  for i in $@; do
    echo -e "Utfører «$i».\n"
    $i
  done
fi
