#!/bin/bash
#     kde-l10n-synk.sh – Synkronisering av omsettingar frå og til KDE.
#     Copyright © 2007–2008 Karl Ove Hufthammer <karl@huftis.org>.
# 
#     This program is free software: you can redistribute it and/or modify
#     it under the terms of the GNU General Public License as published by
#     the Free Software Foundation, either version 3 of the License, or
#     (at your option) any later version.
# 
#     This program is distributed in the hope that it will be useful,
#     but WITHOUT ANY WARRANTY; without even the implied warranty of
#     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#     GNU General Public License for more details.
# 
#     You should have received a copy of the GNU General Public License
#     along with this program.  If not, see <http://www.gnu.org/licenses/>.

## MÅ FIKSAST: Oppdatering av «docs/» i vår SVN (bilde og sånt).

#export KDEREPO=~/utvikling/kde-svn
#export SLNREPO=~/utvikling/skolelinux/i18n
export KDEREPO=/scratch/kde-svn
export SLNREPO=/scratch/sln/i18n
export KOMP="$SLNREPO/skript/kompendium"
versjon="kde4/summit"
export PATH=$SLNREPO/$versjon/pology/scripts:$PATH
sprak="nn nb se"
hjelpetekstar="nei" # Skal hjelpetekstane òg genererast og oppdaterast?
kjapp="ja" # Skal me ta ei kjapp oppdatering eller ei meir grundig?

if [ $kjapp = "ja" ]
then
  operasjon="scatter,merge"
else
  operasjon="scatter,merge"
fi
if [ $hjelpetekstar = "ja" ]
then
  mapper="summit/messages summit/docmessages"
else
  mapper="summit/messages"
fi


function leggtilfjern
{
  nye=$(svn status | grep "^\?" | cut -c8-)
  fjerna=$(svn status | grep "^\!" | cut -c8-)
  if [ -n "$nye" ]; then colorsvn add $nye; fi
  if [ -n "$fjerna" ]; then colorsvn delete $fjerna; fi
}

function oppdater_pot
{
# Oppdater frå KDE-SVN
echo "*** Hentar nyaste malfiler ***"
    cd $SLNREPO/$versjon
    colorsvn up templates
} # Slutt oppdater_pot



function lag_kompendium
{
# Hentar oppdaterte omsettingsfiler frå Skulelinux-SVN
echo -e "\n*** Hentar oppdaterte omsettingsfiler frå Skulelinux-SVN ***\n"
cd $SLNREPO/$versjon
svn up

mkdir -p "$KOMP"

# Lagar kompendium
  for sprk in $sprak
  do
      echo -e "\n*** Lagar kompendium for språk $sprk ***\n"
      cd $SLNREPO/$versjon/$sprk/summit/messages
      find $SLNREPO/$versjon/$sprk $SLNREPO/openoffice/po/$sprk $SLNREPO/other/$sprk -name '*.po' -print0 | xargs -0 msgcat --use-first --output-file="$KOMP/${sprk}.po"
      msgattrib --clear-obsolete --output-file="$KOMP/${sprk}.po" "$KOMP/${sprk}.po"
      msgattrib --translated --set-fuzzy --no-location --no-wrap --output-file="$KOMP/${sprk}.po" "$KOMP/${sprk}.po"
      sed -i '/^#, fuzzy$/{n;/^msgctxt "[^ ]\+ OF TRANSLATORS"$/{n;n;s/".*"/""/}}' "$KOMP/${sprk}.po"
  done
} # Slutt lag_kompendium



function hent_fra_kde3
{
# Henter gamle omsettingsfiler frå KDE 3, og overfører dei til KDE 4.
echo "*** Overfører eventuelle omsettingsfiler frå KDE 3 til KDE 4 ***"
    cd $SLNREPO/$versjon
    colorsvn up templates
    for sprk in $sprak
    do
      colorsvn up $sprk
      for mappe in $mapper
      do
        cd $SLNREPO/$versjon/templates
        find $mappe -name '*.pot' | while read pot
        do
          po=${pot%t}
          fil=$(basename $po)
          adresse=$(dirname $po)
          for kde3vers in kde3-stable kde3-trunk
          do
            if [ ! -e ../$sprk/$po ]
              then
                nyfil=$(find $SLNREPO/$kde3vers/$sprk/$(basename $(echo $mappe)) -name "$fil")
              if [ ! -z "$nyfil" ]
              then
                echo "Kopierer omsetting av $fil frå $kde3vers ($nyfil)."
                mkdir -p "$(dirname "$SLNREPO/$versjon/$sprk/$adresse/$fil")"
                colorsvn add "$(dirname "$SLNREPO/$versjon/$sprk/$adresse/$fil")"
                colorsvn copy $nyfil $SLNREPO/$versjon/$sprk/$adresse/$fil
              fi
            fi
          done # for kde3vers
        done # find $mappe
      done # for mappe
    done # for sprk
    colorsvn commit -m "Overført gamle omsettingar frå KDE 3." $SLNREPO/$versjon/{nb,nn,se}
} # Slutt hent_fra_kde3


function oppdater_po
{
# Hentar oppdaterte omsettingsfiler frå Skulelinux-SVN
echo -e "\n*** Hentar oppdaterte omsettingsfiler frå Skulelinux-SVN ***\n"
cd $SLNREPO/$versjon
svn up

# Ting eg har gløymt å melda inn
  for sprk in $sprak
  do
    for mappe in $mapper
    do
      cd $SLNREPO/$versjon/$sprk/$mappe
      colorsvn commit -m "Omsett meir."
      leggtilfjern
      colorsvn commit -m "Ny omsetting."
    done
  done

# Oppdaterer omsettingsfiler og sånt i Skulelinux
  cd $SLNREPO/$versjon
  sh ./scripts/process_orphans.sh
  for sprk in $sprak
  do
    if [ $sprk = "nn" ]; then
        langsprk="Norwegian Nynorsk"
    elif [ $sprk = "nb" ]; then
        langsprk="Norwegian Bokmål"
    elif [ $sprk = "se" ]; then
        langsprk="Northern Sami"
    fi
    find $sprk -name '*.po' | while read fil
    do
      namn=$(basename ${fil%.po})
      sed -i -r -e '1,/^msg/{/^#[^@]*$/d}' -e '1,/^msg/{/^#[^>]*$/d}' -e "1,1s/(.*)/# Translation of $namn to $langsprk\n#\n\1/" $fil
    done
  done
  find . -name '*.po' | xargs sed -i -e '/^#, fuzzy$/{n;/^msgctxt "[^ ]\+ OF TRANSLATORS"$/{n;n;s/".*"/""/}}' \
                                     -e '/^msgctxt "[^ ]\+ OF TRANSLATORS"$/{n;n;s/ ?, ?/,/g;s/ "$/"/g}'
  sh $SLNREPO/skript/check_po_files
  posieve.py set_header -screate -s'field:Language-Team:Norwegian Nynorsk <i18n-nn@lister.ping.uio.no>' nn
  posieve.py set_header -screate -s'field:Language-Team:Norwegian Bokmål <i18n-nb@lister.ping.uio.no>' nb
  posieve.py set_header -screate -s'field:Language-Team:Northern Sami <i18n-sme@lister.ping.uio.no>' se
  posieve.py set_header -screate -s'field:Plural-Forms:nplurals=2; plural=n != 1;' nb nn se
  posieve.py normalize_header nb nn se
  colorsvn commit -m 'Automatiske rettingar (flytting/sletting av filer, fiksing av filhovud, og liknande).'
  for sprk in $sprak
  do
    cd $SLNREPO/$versjon/$sprk
    echo -e "\n*** Oppdaterer omsettingsfiler for $versjon og språk $sprk ***\n"
    for mappe in $mapper
    do
      cd $SLNREPO/$versjon/$sprk/$mappe
      echo "--- Feilfiksing for $mappe og språk $sprk ---"
      find -name "*.po" -type f | while read fil
      do
          endra=$(sed -f $SLNREPO/skript/feilfiksar $fil)
          echo "$endra" > $fil
          if [ "$fil" = './kdeedu/blinken.po' ]; then # I blinken skal ikkje «...» endrast til «…».
            sed -i 's/…/.../g' $fil
          fi
      done # Ferdig med .po-fil
      colorsvn commit -m "Automatisk formatering, samt retting av vanlige feil."

      echo "--- Oppdatering for $mappe og språk $sprk ---"
      find -name "*.po" -type f | while read po
      do
	    pot=../../../templates/$mappe/${po%.po}.pot
		datopo=$(grep '^"POT-Creation-Date:' $po)
		datopot=$(grep '^"POT-Creation-Date:' $pot)
        if [[ -e "$pot" && ("$datopo" != "$datopot" || "$kjapp" != "ja") ]]; then
			      echo "$po må oppdaterast:"
            msgmerge --verbose --update --previous --backup=none --compendium "$KOMP/${sprk}.po" --silent $po $pot
            porewrap --no-wrap $po
        fi
      done # Ferdig med .po-fil
    done # Ferdig med mappe
  done # Slutt sprk
  cd $SLNREPO/$versjon
  colorsvn commit -m "Oppdaterte omsettingsfiler basert på siste malfiler."
} # Slutt oppdater_po


function magic_po
{
echo -e "\n*** Hentar oppdaterte omsettingsfiler frå Skulelinux-SVN ***\n"
cd $SLNREPO/$versjon
svn up
# Språvask omsettingsfiler.
# Førebels har berre nynorsk har ei fornuftig språkvaskingsfil.
  for sprk in nn
  do
    echo -e "\n*** Språkvaskar omsettingsfiler for $versjon og språk $sprk ***\n"
    cd $SLNREPO/$versjon/$sprk
    find -name "*.po" -type f | while read fil
     do
       $SLNREPO/skript/magicpo/magicpo --silent --nomarkfuzzy --nolasttrans -f $SLNREPO/openoffice/utils/MagicPO/dictionaries/magicpo-${sprk}-spraakvask.dict $fil
       msgmerge --silent --backup=none --previous -U $fil $SLNREPO/$versjon/templates/${fil}t
     done # Ferdig med .po-fil
  done # Slutt sprk
  cd $SLNREPO/$versjon
  colorsvn commit -m "Automatisk språkvask ved hjelp av MagicPO."
} # Slutt magic_po

function tilbake_po
{
    rsync_val="--exclude-from=$SLNREPO/skript/ekskluderingsfil --recursive --delete"

    echo -e "\n*** Hentar nyaste omsettingsfiler, malar og skript ***\n"
    colorsvn up $KDEREPO/{trunk,branches/stable}/l10n-kde4/{scripts,templates,nb,nn,se}
    colorsvn up $KDEREPO/trunk/l10n-support/{scripts,templates,pology,nb,nn,se}
    colorsvn up -N $KDEREPO/trunk/l10n-support/

    # Flytting og sånt:
    cd $KDEREPO/trunk/l10n-kde4
    sh ./scripts/process_orphans.sh
    cd $KDEREPO/branches/stable/l10n-kde4
    sh ./scripts/process_orphans.sh

    for sprk in $sprak
    do
      echo -e "\n*** Legg til / slettar for $versjon og språk $sprk i KDEREPO ***\n"
      cd $KDEREPO/trunk/l10n-support/$sprk
      rsync $rsync_val $SLNREPO/$versjon/$sprk/ $KDEREPO/trunk/l10n-support/$sprk
      if [ $sprk = "nn" ]; then
        find . -name '*.po' -type f -print0 | xargs -0 sed -r -i -f $SLNREPO/skript/formfeil
      fi
      leggtilfjern
      if [ $sprk = "nn" ]; then
          langsprk="Norwegian Nynorsk"
      elif [ $sprk = "nb" ]; then
          langsprk="Norwegian Bokmål"
      elif [ $sprk = "se" ]; then
          langsprk="Northern Sami"
      fi
      cd $KDEREPO/trunk/l10n-support/
      posummit.py scripts/messages.summit $sprk $operasjon
      if [ $hjelpetekstar = "ja" ]
      then
        posummit.py scripts/docmessages.summit $sprk $operasjon
      fi
      
      cd $KDEREPO/trunk/l10n-support/$sprk
      leggtilfjern
      cd ..
      colorsvn commit -m "Updated $langsprk translations for summit." $sprk
      
      cd $KDEREPO/trunk/l10n-kde4/$sprk
      rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/data .
      rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/scripts .
      leggtilfjern
      cd ..
      colorsvn commit -m "Updated $langsprk translations for kde4-trunk." $sprk
      
      cd $KDEREPO/branches/stable/l10n-kde4/$sprk
      rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/data .
      rsync $rsync_val $SLNREPO/$versjon/$sprk/summit/scripts .
      leggtilfjern
      cd ..
      colorsvn commit -m "Updated $langsprk translations for kde4-stable." $sprk
    done
} # Slutt tilbake_po

function lag_dok
{
    colorsvn up $KDEREPO/{trunk,branches/stable}/l10n-kde4/{documentation,*/docs}
    for sprk in $sprak
    do
      echo -e "\n*** Lagar hjelpefiler for trunk og språk $sprk ***\n"
      cd $KDEREPO/trunk/l10n-kde4
      ./scripts/update_xml $sprk
      colorsvn commit -m "Updated $langsprk translations for documentation (kde4-trunk)." $sprk/docs

      echo -e "\n*** Lagar hjelpefiler for stable og språk $sprk ***\n"
      cd $KDEREPO/branches/stable/l10n-kde4
      ./scripts/update_xml $sprk
      colorsvn commit -m "Updated $langsprk translations for documentation (kde4-stable)." $sprk/docs
    done
} # Slutt tilbake_po

echo -e "kde-l10n-synk.sh\n\
    Copyright © 2007 Karl Ove Hufthammer <karl@huftis.org>.
    This program comes with ABSOLUTELY NO WARRANTY; for details,
    see the source code. This is free software, and you are welcome
    to redistribute it under certain conditions; again, see the
    source code for details.\n"

val=$1
if [ -z "$val" ]; then
  echo -e "Bruk:\n\
  oppdater_pot   Oppdater malane frå KDE-SVN og kopier dei over til Skulelinux.\n\
  hent_fra_kde3  Hent eventuelle omsettingsfiler frå KDE 3 og overfør dei til KDE 4.\n\
  oppdater_po    Oppdater omsettingsfilene i Skulelinux-SVN med nytt frå malane.\n\
  lag_kompendium Lagar kompendium over alle tekstane i omsettingane.
  magic_po       Køyrer MagicPO-baserte språkvaskingsskript på omsettingane.\n\
  tilbake_po     Overfør omsettingsfilene tilbake til KDE-SVN.\n\
  lag_dok        Lag og send inn hjelpefiler basert på omsettingane i KDE-SVN.\n\
  alt            Gjer alt ovanfor (i rekkjefølgje)."
elif [ $val = alt ]; then
  oppdater_pot
  lag_kompendium
  oppdater_po
  hent_fra_kde3
#  magic_po
  tilbake_po
if [ $hjelpetekstar = "ja" ]
then
  lag_dok
fi
else
  for i in $@; do
    echo -e "Utfører «$i».\n"
    $i
  done
fi
