library(ggplot2) # Load graphing package.

people=read.table("translation-statistics.dat", sep="\t", col.names=c("Person","Year")) # Read translations statistics.

# Sort graph first by last year the person worked, and then by the first year:
people$Person=with(people,reorder(Person,Year,min)) # Second sort key: First year worked.
people$Person=with(people,reorder(Person,Year,max)) # First sort key: Last year worked.

# Save the plot.
# You might want tlay around with the ‘type’ and ‘antialias’ arguments to improve the rendering.
png("translation-statistics.png",width=800,height=600)

# Could have used simply
# qplot(Year, Person, data=people, geom="tile")
# but let’s instead do some customising
# to improve the look of the graph.
ggplot(people, aes(x=Year,y=Person)) +              # Set up the graph.
  geom_vline(xintercept=seq(min(people$Year)-.5,
                        max(people$Year)+.5),
                        colour="grey90") +          # Draw vertical lines for each year.
  geom_tile() +                                     # Draw the boxes.
  scale_x_continuous(breaks=unique(people$Year)) +  # Show labels for *all* the years on the x-axis.
  xlab(NULL) + ylab(NULL) +                         # Remove the axis labels.
  theme_bw() +                                      # Remove the grey background.
  opts(panel.grid.major=theme_blank(),              # Remove the default grid lines, tick marks and border.
       panel.grid.minor=theme_blank(),
       axis.ticks=theme_blank(),
       panel.border=theme_blank())

dev.off() # Save the graph.
