#!/bin/sh
# Create graph showing which year the translators have been working on translations.

# Folder containing the PO files.
folder=~/utvikling/skolelinux/i18n/kde4/summit/nn/summit/

# Explanation:
# Find all the PO files.
# Print only the author lines.
# Print one line for each year the author has worked (assumes the ‘can_header’ sieve has already been run).
# Print the lines on the format ‘Author<tab>Year’ (the ‘substr’ is just a hack to get rid of any trailing periods).
# Filter out any ‘FIRST AUTHOR<tab>YEAR’ entries.
# Sort the entries.
# Remove duplicate entries, and save the result.
find "$folder" -name '*.po' \
| xargs grep -h '^# .*<.*@' \
| awk 'BEGIN {FS=","} { i=2; while (i<=NF){ print $1"\t"substr($i,1,5); i++ } }' \
| sed 's/^# //;s/ <[^>]\+>//' \
| grep -v 'FIRST AUTHOR' \
| sort \
| uniq > translation-statistics.dat

# Run an R script that generates the graph and saves it as ‘translation-statistics.png’.
R --vanilla --slave -f translation-graph.R

# Optionally make the PNG more Web friendly with Adam7 interlacing.
OPTIPNG=$(command -v optipng)
OPTIPNG_EXISTS=$?
if [ $OPTIPNG_EXISTS -eq 0 ]
then
  $OPTIPNG -i1 translation-statistics.png
fi
